package com.sd.business.service.production.impl;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ruoyi.common.core.domain.BaseIdPo;
import com.ruoyi.common.core.domain.BasePo;
import com.ruoyi.common.core.domain.BaseSelectDto;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.sd.business.entity.order.enums.OrderStatusEnum;
import com.sd.business.entity.order.po.OrderInfo;
import com.sd.business.entity.order.po.OrderPackageBom;
import com.sd.business.entity.order.po.OrderSku;
import com.sd.business.entity.production.dto.ProductionOrderDto;
import com.sd.business.entity.production.dto.RapidPackagingDto;
import com.sd.business.entity.production.dto.SetTagDto;
import com.sd.business.entity.production.po.ProductionWorkOrder;
import com.sd.business.entity.production.vo.ProductionOrderDetailVo;
import com.sd.business.entity.production.vo.ProductionOrderScheduleVo;
import com.sd.business.entity.production.vo.ProductionOrderVo;
import com.sd.business.entity.statement.dto.StatementOfAccountDto;
import com.sd.business.mapper.production.ProductionOrderMapper;
import com.sd.business.service.bom.BomSpecService;
import com.sd.business.service.inventory.InventoryFinishedService;
import com.sd.business.service.order.OrderPackageBomService;
import com.sd.business.service.order.OrderService;
import com.sd.business.service.order.OrderSkuService;
import com.sd.business.service.production.ProductionOrderService;
import com.sd.business.service.production.ProductionWorkOrderService;
import com.sd.business.service.sku.SkuSpecService;
import com.sd.business.service.statement.StatementOfAccountService;
import com.sd.framework.util.Assert;
import com.sd.mq.enums.WorkOrderStatusEnum;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

@Service
public class ProductionOrderServiceImpl implements ProductionOrderService {

    @Autowired
    private ProductionOrderMapper productionOrderMapper;

    @Autowired
    private SkuSpecService skuSpecService;

    @Autowired
    private OrderService orderService;

    @Autowired
    private OrderSkuService orderSkuService;

    @Autowired
    private BomSpecService bomSpecService;

    @Autowired
    private OrderPackageBomService orderPackageBomService;

    @Autowired
    private StatementOfAccountService statementOfAccountService;

    @Autowired
    private InventoryFinishedService inventoryFinishedService;

    @Autowired
    private ProductionWorkOrderService productionWorkOrderService;

    @Override
    public Page<ProductionOrderVo> getPage(ProductionOrderDto dto) {

        IWrapper<ProductionOrderVo> wrapper = IWrapper.getWrapper();
        wrapper.in("oi", OrderInfo::getStatus, OrderStatusEnum.IN_PRODUCTION.getKey(), OrderStatusEnum.COMPLETION_PRODUCTION.getKey());
        wrapper.like("oi", OrderInfo::getWlnCode, dto.getWlnCode());
        wrapper.eq("oi", OrderInfo::getDepartmentId, dto.getDepartmentId());
        wrapper.in("oi", OrderInfo::getTag, dto.getTagList());
        wrapper.ge("oi", OrderInfo::getDeliveryTime, dto.getBeginTime());
        wrapper.le("oi", OrderInfo::getDeliveryTime, dto.getEndTime());
        wrapper.eq("oi", OrderInfo::getStatus, dto.getStatus());
        if (StrUtil.isNotBlank(dto.getCode())) {
            wrapper.and(q -> q.like("oi", OrderInfo::getCode, dto.getCode()).or().like("oi", OrderInfo::getWlnCode, dto.getCode()));
        }

        Page<ProductionOrderVo> page = productionOrderMapper.getPage(dto.getPage(), wrapper);
        List<ProductionOrderVo> records = page.getRecords();
        if (records.isEmpty()) {
            return page;
        }

        // 赋值进度
        List<Long> orderIdList = records.stream().map(ProductionOrderVo::getOrderId).collect(Collectors.toList());
        Map<Long, List<ProductionOrderScheduleVo>> map = productionWorkOrderService.getProductionOrderScheduleMap(orderIdList);
        for (ProductionOrderVo record : records) {
            record.setProductionOrderScheduleVoList(map.getOrDefault(record.getOrderId(), Collections.emptyList()));
        }

        return page;
    }

    @Override
    public void setTag(SetTagDto dto) {
        orderService.update(q -> q.in(BaseIdPo::getId, dto.getOrderIdList()).set(OrderInfo::getTag, dto.getTag()));
    }

    @Override
    public ProductionOrderDetailVo detail(BaseSelectDto dto) {
        Long id = dto.getId();
        Assert.notNull(id, "订单id不能为空");

        List<ProductionOrderDetailVo.SkuSpec> skuSpecList = getSkuSpecList(id);
        List<ProductionOrderDetailVo.OrderPackage> packageList = getPackageList(id);

        ProductionOrderDetailVo vo = new ProductionOrderDetailVo();
        vo.setSkuSpecList(skuSpecList);
        vo.setPackageList(packageList);
        return vo;
    }

    @Override
    public synchronized void rapidPackaging(RapidPackagingDto dto) {
        Long orderId = dto.getOrderId();
        OrderInfo orderInfo = orderService.getById(orderId);
        if (!orderInfo.getStatus().equals(OrderStatusEnum.IN_PRODUCTION.getKey())) {
            throw new ServiceException("订单已被处理");
        }

        List<OrderPackageBom> orderPackageBomList = dto.getPackageList();
        orderPackageBomList.forEach(item -> item.setOrderId(orderId));
        orderPackageBomService.editLinked(orderPackageBomList, OrderPackageBom::getOrderId, orderId);

        Date date = new Date();
        orderService.update(q -> q
                .eq(BaseIdPo::getId, orderId)
                .set(OrderInfo::getStatus, OrderStatusEnum.COMPLETION_PRODUCTION.getKey())
                .set(OrderInfo::getShippingTime, date)
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
                .set(BasePo::getUpdateTime, date));

        // 订单生产完成时，一键完成生产任务
        productionWorkOrderService.update(q -> q
                .eq(ProductionWorkOrder::getOrderId, orderId)
                .notIn(ProductionWorkOrder::getStatus, WorkOrderStatusEnum.PRODUCTION_ANOMALY.getKey(), WorkOrderStatusEnum.RESCHEDULE.getKey())
                .set(ProductionWorkOrder::getStatus, WorkOrderStatusEnum.PRODUCTION_COMPLETION.getKey())
                .set(ProductionWorkOrder::getCompleteTime, date)
                .set(BasePo::getUpdateTime, date)
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId()));

        // 生成对账单
        StatementOfAccountDto statement = new StatementOfAccountDto();
        statement.setDepartmentId(orderInfo.getDepartmentId());
        statement.setOrderIdList(Collections.singletonList(orderInfo.getId()));
        statementOfAccountService.add(statement);

        // 生产入库
        inventoryFinishedService.productionWarehousing(Collections.singletonList(orderId));

        // 库存销售出库
        inventoryFinishedService.saleOutOfWarehouse(Collections.singletonList(orderId));

    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void finishOrder(Long id) {
        OrderInfo orderInfo = orderService.getById(id);
        if (orderInfo == null) {
            throw new ServiceException("未找到订单");
        }
        if (!orderInfo.getStatus().equals(OrderStatusEnum.IN_PRODUCTION.getKey())) {
            throw new ServiceException("订单已被处理");
        }

        Date date = new Date();
        orderService.update(q -> q
                .eq(BaseIdPo::getId, id)
                .set(OrderInfo::getStatus, OrderStatusEnum.COMPLETION_PRODUCTION.getKey())
                .set(OrderInfo::getShippingTime, date)
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
                .set(BasePo::getUpdateTime, date));

        // 订单生产完成时，一键完成生产任务
        productionWorkOrderService.update(q -> q
                .eq(ProductionWorkOrder::getOrderId, id)
                .notIn(ProductionWorkOrder::getStatus, WorkOrderStatusEnum.PRODUCTION_ANOMALY.getKey(), WorkOrderStatusEnum.RESCHEDULE.getKey())
                .set(ProductionWorkOrder::getStatus, WorkOrderStatusEnum.PRODUCTION_COMPLETION.getKey())
                .set(ProductionWorkOrder::getCompleteTime, date)
                .set(BasePo::getUpdateTime, date)
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId()));

        // 生成对账单
        StatementOfAccountDto statement = new StatementOfAccountDto();
        statement.setDepartmentId(orderInfo.getDepartmentId());
        statement.setOrderIdList(Collections.singletonList(orderInfo.getId()));
        statementOfAccountService.add(statement);

        // 生产入库
        inventoryFinishedService.productionWarehousing(Collections.singletonList(id));

        // 库存销售出库
        inventoryFinishedService.saleOutOfWarehouse(Collections.singletonList(id));
    }

    private List<ProductionOrderDetailVo.SkuSpec> getSkuSpecList(Long orderId) {

        // 获取订单sku规格
        List<OrderSku> orderSkuList = orderSkuService.list(q -> q.eq(OrderSku::getOrderId, orderId));

        // 相同订单sku规格合并，数量相加
        List<ProductionOrderDetailVo.SkuSpec> skuSpecList = new ArrayList<>(
                orderSkuList.stream()
                        .map(item -> {
                            ProductionOrderDetailVo.SkuSpec skuSpec = new ProductionOrderDetailVo.SkuSpec();
                            skuSpec.setSkuSpecId(item.getSkuSpecId());
                            skuSpec.setQuantity(item.getQuantity());
                            return skuSpec;
                        })
                        .collect(Collectors.toMap(
                                ProductionOrderDetailVo.SkuSpec::getSkuSpecId,
                                Function.identity(),
                                (v1, v2) -> {
                                    v1.setQuantity(v1.getQuantity().add(v2.getQuantity()));
                                    return v1;
                                })
                        ).values());

        // 赋值订单sku规格品号以及品名
        skuSpecService.attributeAssign(skuSpecList, ProductionOrderDetailVo.SkuSpec::getSkuSpecId, (item, skuSpec) -> {
            item.setSkuSpecName(skuSpec.getName());
            item.setSkuSpecCode(skuSpec.getCode());
        });

        return skuSpecList;
    }

    private List<ProductionOrderDetailVo.OrderPackage> getPackageList(Long orderId) {

        List<OrderPackageBom> list = orderPackageBomService.list(q -> q.eq(OrderPackageBom::getOrderId, orderId));

        List<ProductionOrderDetailVo.OrderPackage> orderPackageList = list.stream().map(item -> {
            ProductionOrderDetailVo.OrderPackage orderPackage = new ProductionOrderDetailVo.OrderPackage();
            orderPackage.setBomSpecId(item.getBomSpecId());
            orderPackage.setQuantity(item.getQuantity());
            orderPackage.setCostPrice(item.getCostPrice());
            orderPackage.setInternalSellingPrice(item.getInternalSellingPrice());
            return orderPackage;
        }).collect(Collectors.toList());

        // 赋值品名 品号 销售单价 成本价
        bomSpecService.attributeAssign(orderPackageList, ProductionOrderDetailVo.OrderPackage::getBomSpecId, (item, bomSpec) -> {
            item.setBomSpecCode(bomSpec.getCode());
            item.setBomSpecName(bomSpec.getName());
        });

        return orderPackageList;
    }

}
